/**
 * PacMan for J2ME Devices
 * CS 327 - Design Project, Fall 2002
 * University of Illinois, Urbana-Champaign
 * 
 * file: PacMantest.java
 * Used for unit testing of Pacman code.
 **/

//----------------------------------------------------------------------------//

import junit.framework.* ;

public class PacMantest extends TestCase
{
    protected PacmanActor pactor;
    protected PacmanActor pactor2;
    protected GridNode grid;
    protected GridNode grid2;
    protected GridNode grid3;
    protected GhostActor gactor;
    protected int mouth;
    protected int pellet;
    protected GameBoard board;
    
    /**
     * Initializes variables used in testing.
     */
    public void setUp()
    {
	grid = new GridNode(2, 3); 
	grid2 = new GridNode(5, 7);
	grid3 = new GridNode(4, 4);
    	pactor = new PacmanActor(grid);
	mouth = 320;
	pellet = 0;
	gactor = new GhostActor(grid2, pactor);
	board = new GameBoard();
    }
    
    /**
     * Tests PacmanActor initialization.
     */
    public void testPacman_initial()
    {
	assertEquals(pactor.getMouthAngle_test(), mouth);
	assertEquals(pactor.getPelletsEaten(), pellet);
	assertEquals(pactor.getGridX_test(), grid.getGridX_test());
	assertEquals(pactor.getGridY_test(), grid.getGridY_test());
	assertEquals(pactor.getNextPixelX_test(), grid.pixelCenterX());
	assertEquals(pactor.getNextPixelY_test(), grid.pixelCenterY());
	assertEquals(pactor.getPixelX_test(), grid.pixelCenterX());
	assertEquals(pactor.getPixelY_test(), grid.pixelCenterY());
	assertEquals(pactor.getDirection_test(), 0);
	System.out.println( "This method tests the initialization of pacman." );
    }
    
    /**
     * Tests GhostActor initialization.
     */
    public void testGhost_initial()
    {
	assertEquals(gactor.getGridX_test(), grid2.getGridX_test());
	assertEquals(gactor.getGridY_test(), grid2.getGridY_test());
	assertEquals(gactor.getTarget_test(), pactor);
	assertEquals(gactor.getNextPixelX_test(), grid.pixelCenterX());
	assertEquals(gactor.getNextPixelY_test(), grid.pixelCenterY());
	assertEquals(gactor.getPixelX_test(), grid.pixelCenterX());
	assertEquals(gactor.getPixelY_test(), grid.pixelCenterY());		
	System.out.println("Tests that ghostactors are initialized properly.");

    }
    
    /**
     * Tests that game is reinitialized in terms of PacmanActor
     */
    public void testRestart()
    {
	pactor.goToNode_test(grid2);
	pactor.restart();
	assertEquals(pactor.getMouthAngle_test(), mouth);
	assertEquals(pactor.getPelletsEaten(), pellet);
	assertEquals(pactor.getGridX_test(), grid.getGridX_test());
	assertEquals(pactor.getGridY_test(), grid.getGridY_test());
	assertEquals(pactor.getNextPixelX_test(), grid.pixelCenterX());
	assertEquals(pactor.getNextPixelY_test(), grid.pixelCenterY());
	assertEquals(pactor.getPixelX_test(), grid.pixelCenterX());
	assertEquals(pactor.getPixelY_test(), grid.pixelCenterY());
	assertEquals(pactor.getDirection_test(), 0);
	System.out.println( " This method tests to see whether the PacmanActor "
			    + "is reinitialized properly." );
    }
    
    /**
     * Tests that PacmanActor can change positions.
     */
    public void testPacman_node()
    {
	pactor.goToGridNode_test(grid2);
        assertEquals(pactor.getNextPixelX_test() , 5 );
        assertEquals(pactor.getNextPixelY_test() , 7 );
	System.out.println( "This method tests that pacman moves as instructed." );
    }
    
    /**
     * Tests that GhostActor can change positions.
     */
    public void testGhost_node()
    {
	gactor.goToGridNode_test(grid);
        assertEquals(gactor.getNextPixelX_test() , 2 );
        assertEquals(gactor.getNextPixelY_test() , 3 );
	System.out.println( "This method tests that ghosts move as instructed." );
    }
    
    /**
     * Tests that PacmanActor is killed according to GridNode
     */
    public void testPacman_death()
    {
	pactor.goToGridNode_test(grid2);
	gactor.goToGridNode_test(grid2);
	assertEquals(pactor.getNextPixelX_test(), gactor.getNextPixelX_test());
	assertEquals(pactor.getNextPixelY_test(), gactor.getNextPixelY_test());
	board.addGridNode(grid);
	board.addGridNode(grid2);
	board.addGhost(gactor);
	board.setPacman(pactor);
	board.advanceFrame_test();
	assertEquals(true, board.state_test() == 1);
	System.out.println( "This method tests that pacman can be killed." );
    }
    
    /**
     * Tests that PacmanActor knows it eats pellets on Gameboard and that
     * board knows pellets are no longer there.
     */
    public void testPelletEating()
    {
	board.addGridNode(grid);
	board.setPacman(pactor);
	assertEquals(0, pactor.getPelletsEaten());
	pactor.advanceFrame();
	assertEquals(1, pactor.getPelletsEaten());
	assertEquals(0, grid.getPellet());
	System.out.println("This method tests that pacman knows he ate a pellet "+
			   "the GridNode knows a pellet was eaten");
    }
    
    /**
     * Tests that we are using correct Pacman Actors
     */
    public void testGetPacman() 
    {
	pactor2 = pactor;
	board.setPacman(pactor);
	assertEquals(board.getPacman(), pactor2);
	System.out.println("This method tests that we are using correct Pacman Actors");
    }
    
    /**
     * Tests that score is incremented properly.
     */
    public void testScore()
    {
	pactor.goToGridNode_test(grid2);
	gactor.goToGridNode_test(grid2);
	board.addGridNode(grid);
	board.addGridNode(grid2);
	board.addGhost(gactor);
	board.setPacman(pactor);
	pactor.invincible_ = true;
	board.advanceFrame_test();
	assertEquals(5, pactor.score_);
	pactor.goToNode_test(grid3);
	board.addGridNode(grid3);
	board.setPacman(pactor);
	pactor.advanceFrame();
	pactor.advanceFrame();
	assertEquals(6, pactor.score_);
	System.out.println("This method tests that score is incrememented properly.");
    }
    
    /**
     * Tests that lives are decremented properly.
     */
    public void testLives()
    {
	pactor.goToGridNode_test(grid2);
	gactor.goToGridNode_test(grid2);
	board.addGridNode(grid);
	board.addGridNode(grid2);
	board.addGhost(gactor);
	board.setPacman(pactor);
	assertEquals(pactor.getLivesLeft(),2);
	board.advanceFrame_test();
	pactor.restart();
	assertEquals(true, board.state_test() == 1);
	assertEquals(pactor.getLivesLeft(),1);
	pactor.goToGridNode_test(grid3);
	gactor.goToGridNode_test(grid3);
	board.addGridNode(grid3);
	board.advanceFrame_test();
	pactor.restart();
	assertEquals(pactor.getLivesLeft(), 0);
    }
}
